#!/usr/bin/env python
# vim: set ts=8 sw=4 sts=4 et:
# ======================================================================
# Copyright (C) 2008-2016 Walter Doekes (wdoekes) at OSSO B.V.
# This file is part of Pyl10n.
#
# Pyl10n is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyl10n is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyl10n.  If not, see <http://www.gnu.org/licenses/>.
# ======================================================================
from __future__ import unicode_literals

import io
import json
import os

# set default locale path to ../locale/
_locale_path = os.path.join(os.path.dirname(__file__), 'locale')
# set default locale to C
_current_locale_callable = (lambda: 'C')


def setlocale(locale):
    setlocalefunc(lambda: locale)


def setlocalefunc(callable):
    global _current_locale_callable
    _current_locale_callable = callable


def setlocalepath(dirname):
    '''
    Set directory containing message catalogs.

    Message catalogs will be expected at the filename
    dirname/locale/category, where locale is a locale
    name and category is a locale facet such as
    LC_NUMERIC.
    '''
    global _locale_path
    _locale_path = dirname


def localeconv(locale=None):
    locale = locale or _get_locale()
    ret = {}
    ret.update(_get_category(locale, 'LC_MONETARY'))
    ret.update(_get_category(locale, 'LC_NUMERIC'))
    # We've removed trailing zeroes at generation time
    assert 0 not in ret['grouping'] and 0 not in ret['mon_grouping']
    return ret


def localeconv_by_category(category, locale=None):
    locale = locale or _get_locale()
    assert category in (
        'LC_ADDRESS', 'LC_MEASUREMENT', 'LC_MONETARY', 'LC_NAME',
        'LC_NUMERIC', 'LC_PAPER', 'LC_TELEPHONE', 'LC_TIME')
    return _get_category(locale, category)


def _get_category(locale, category):
    if locale not in _get_category._cache:
        _get_category._cache[locale] = {}
    catcache = _get_category._cache[locale]
    if category not in catcache:
        file = None
        try:
            file = io.open(os.path.join(_locale_path, locale, category), 'r',
                           encoding='utf-8')
            ret = json.load(file)
            assert type(ret) == dict
            catcache[category] = ret
        except Exception as e:
            from sys import stderr
            stderr.write('pyl10n: loading locale category: %s\n' % e)
            catcache[category] = {}
        finally:
            if file is not None:
                file.close()
    return catcache[category]


_get_category._cache = {}


def _get_locale():
    global _current_locale_callable
    return _current_locale_callable()


def pyl10n_core_test():
    # TODO: create tests :)
    pass



#!/usr/bin/env python
# vim: set ts=8 sw=4 sts=4 et:
# ======================================================================
# Copyright (C) 2008-2016 Walter Doekes (wdoekes) at OSSO B.V.
# This file is part of Pyl10n.
#
# Pyl10n is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyl10n is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyl10n.  If not, see <http://www.gnu.org/licenses/>.
# ======================================================================


# INTRODUCTION
# ============
# Pyl10n is a thread-safe /locale module/ replacement. Most functions
# behave very similarly to their locale.* counterparts.
#
# Load this file (__init__.py) to get all functionality, e.g.:
# {{{
# import path.to.pyl10n as locale
# }}}
#
# The functions that are currently implemented are the following, the
# (+) marks functions that are not found in the regular locale module.
# Most functions will also accept an optional callable that returns the
# current locale.
#
# FILE pyl10n_core:
#  * localeconv
#  * localeconv_by_category (+)
#  * setlocale
#  * setlocalefunc (+)
#  * setlocalepath (+)
#
# FILE pyl10n_numeric
#  * atof
#  * atoi
#  * currency
#  * dutch_round (+)
#  * format
#  * str
#
# FILE pyl10n_telephone
#  * teldom2string (+)
#  * telint2string (+)
#
# FILE pyl10n_time
#  * format_date (+)
#  * format_datetime (+)
#  * format_time (+)
#  * strftime


# QUICK HOWTO
# ===========
# Before using pyl10n, make sure that the path in pyl10n_core is set up
# correctly for your file layout. Edit the following statement to fit
# your needs:
# {{{
# _locale_path = os.path.join(os.path.dirname(__file__), '..', 'locale')
# }}}
#
# Using pyl10n like locale:
# {{{
# import path.to.pyl10n as locale
# locale.setlocale('nl_NL')
# print(locale.currency(12345.67))
# }}}
#
# Using pyl10n in your threaded app:
# {{{
# import path.to.pyl10n as locale, datetime
# locale.setlocalefunc(some_func_that_returns_locale_for_current_thread)
# print(locale.format_time(datetime.datetime.now()))
# }}}
#
# Quickly switching languages:
# {{{
# import path.to.pyl10n as locale, datetime
# now = datetime.datetime.now()
# print(locale.format_time(now, 'nl_NL'))
# print(locale.format_time(now, 'en_US'))
# }}}


# DIFFERENCES WITH LOCALE
# =======================
# The following differences with the original locale module should be
# observed:
#  * Use only the language/region when specifying the locale.
#    E.g.: ('nl_NL') instead of (LC_NUMERIC, 'nl_NL.UTF-8').
#  * Pyl10n does not load variables like DAY_1 etc. in the global
#    context. You can get them from localeconv_by_category if you need
#    them. (See the pyl10n_time functions for quick shortcuts.)
#  * Python is inconsistent with stripping or not stripping trailing
#    zeroes from lists in locale definitions. Pyl10ngen always strips
#    trailing zeroes.
#  * Extended currency to accept a non-boolean for symbol that will be
#    used as the currency symbol, e.g. currency='EUR '.


# BUGS / MISSING FEATURES
# =======================
# The following things are missing from pyl10n:
#  * Proper regression tests are not done yet.
#  * Address/telephone formatting functions are still missing or very
#    rudimentary.
#  * The locale path is 'hardcoded' relative to __file__ in this file.
#  * The standard is unclear about how to handle [np]_sep_by_space for
#    international sumbols. We assume that the space is an optional
#    fourth character in int_curr_symbol. See
#    http://bugs.python.org/issue1222 for more information.
#  * format(...monetary=True) gets handled by currency(). This means
#    that your format string will be ignored.
#  * Replace 'category' with 'facet' perhaps?
#  * Document setlocalepath (and link its usage to bindtextomain(3))


# WON'T BE IMPLEMENTED
# ====================
# The following things will not be implemented at all:
#  * Character type and collation functions will not be implemented
#    (e.g. strcoll, strxfrm, format_string).
#  * string.letters is not modified when calling setlocale(). (See the
#    previous point.)
from __future__ import print_function, unicode_literals

from .pyl10n_core import *
from .pyl10n_numeric import *
from .pyl10n_telephone import *
from .pyl10n_time import *


def pyl10n_old_test():
    import __builtin__
    import locale
    import json

    for lang in ('nl_NL', 'en_US'):
        print('**** %s ****\n' % (lang,))
        try:
            locale.setlocale(locale.LC_MONETARY, lang + '.utf-8')
            locale.setlocale(locale.LC_NUMERIC, lang + '.utf-8')
        except locale.Error:
            raise ValueError(
                'Need %s.utf-8 locale for this test. Please "apt-get install '
                'language-pack-nl" or similar..' % (lang,))
        setlocale(lang)

        print('%24s|%24s' % ('[locale]', '[pyl10n]'))
        lconv = locale.localeconv()
        pconv = localeconv()
        keys = lconv.keys()
        keys.sort()
        for k in keys:
            print('%24s|%24s <= %s' % (
                __builtin__.str(lconv[k]).decode('utf-8'), pconv[k], k))
        print()

        print('%24s|%24s' % ('[locale]', '[pyl10n]'))

        print('%24s|%24s' % (locale.str(-3.1415), str(-3.1415)))
        for val in (0.7, -0.7, 1234567.89, -1234567.89):
            for monetary in (False, True):
                lval = (
                    locale.format(b'%f', val, True, monetary).decode('utf-8'))
                pval = format('%f', val, True, monetary)

                if not monetary:
                    assert val == locale.atof(lval) and val == atof(pval), \
                            'atof() is broken on value \'%s\'' % (val,)
                print('%24s|%24s' % (lval, pval))

        for val in (0.7, -0.7, 1234567.89, -1234567.89):
            for intl in (False, True):
                lval = locale.currency(val, True, True, intl).decode('utf-8')
                pval = currency(val, True, True, intl)
                print('%24s|%24s' % (lval, pval))
        print()

        if lang == 'nl_NL':
            assert atof('1012,34', allow_grouping=True) == 1012.34
            try:
                atof('123.000', allow_grouping=False)
            except Exception:
                pass
            else:
                assert False, 'atof() should\'ve raised an exception'
        elif lang == 'en_US':
            assert atof('1,012.34', allow_grouping=True) == 1012.34
            try:
                atof('123,000', allow_grouping=False)
            except Exception:
                pass
            else:
                assert False, 'atof() should\'ve raised an exception'

        print('All available locale data:')
        for cat in (
                'LC_ADDRESS', 'LC_MEASUREMENT', 'LC_MONETARY',
                'LC_NAME', 'LC_NUMERIC', 'LC_PAPER', 'LC_TELEPHONE',
                'LC_TIME'):
            print('   %s' % (
                json.dumps(localeconv_by_category(cat), sort_keys=True),))
        print()


def selftest():
    import codecs
    import locale
    import sys

    sys.stdout = (
        codecs.getwriter(locale.getdefaultlocale()[1])(sys.stdout, 'replace'))

    pyl10n_core_test()
    pyl10n_numeric_test()
    pyl10n_telephone_test()
    pyl10n_time_test()

    pyl10n_old_test()



#!/usr/bin/env python
# vim: set ts=8 sw=4 sts=4 et:
# ======================================================================
# Copyright (C) 2008-2016 Walter Doekes (wdoekes) at OSSO B.V.
# This file is part of Pyl10n.
#
# Pyl10n is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyl10n is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyl10n.  If not, see <http://www.gnu.org/licenses/>.
# ======================================================================


# INTRODUCTION
# ============
# Pyl10n is a thread-safe /locale module/ replacement. Pyl10ngen
# converts libc-supplied locale sources files to pickled python files
# usable by pyl10n.
#
# Parsing the libc locale files has been pure guesswork. So sometimes
# the results may not be what the original author intended. See the BUGS
# / NOTES section for a couple of questions that remain.


# QUICK HOWTO
# ===========
# You shouldn't need to run this file. Preconverted languages files can
# be found where these python files are distributed:
# http://code.osso.nl/projects/pyl10n/


# BUGS / NOTES
# ============
#  * Comments might be legal further on the line, but its not used
#    often (only 3 out of 226 files do that).
#    Example: Language file as_IN corrupt: unexpected data:
#     \_ IGNORE;IGNORE;IGNORE;%... (%...)
#  * We only use the categories defines in the CATEGORIES list and not
#    LC_CTYPE/LC_COLLATE.. they are complicated and beyond the scope of
#    pyl10ns intentions right now.
#  * The copy keyword in a category tells us that you should only read
#    that category from the copied file.
#  * There is an 'include' keyword. We don't use it yet.
#  * We strip trailing zeroes from a list (e.g. mon_grouping 3;3;0
#    becomes [3,3]). This is not a bug but a deviation from the
#    C-documentation.

from __future__ import unicode_literals

import io
import itertools
import os
import re

import six

# Read only these categories.. set to None to read all.
CATEGORIES = ('LC_ADDRESS', 'LC_MEASUREMENT', 'LC_MONETARY',
              'LC_NAME', 'LC_NUMERIC', 'LC_PAPER', 'LC_TELEPHONE',
              'LC_TIME')


class ParseException(Exception):
    pass


def parse_args(string, escape_char):
    re_nextarg = re.compile(r'(\s*(%s)\s*)' % '|'.join([
        r'LC_[A-Z]+', r'[A-Za-z_:-]+', r'"[^"]*"', r'-?[0-9]+',
        r'<[0-9A-Za-z_-]+>', r'<[0-9A-Za-z_-]+>..<[0-9A-Za-z_-]+>',
        r'\(<[0-9A-Za-z_-]+>,<[0-9A-Za-z_-]+>\)'
    ]))
    ret = []

    i = 0
    while True:
        m = re_nextarg.match(string, i)
        if not m:
            raise ParseException(
                'Unexpected data: \'\'\'%s<-- HERE -->%s\'\'\'' %
                (string[:i], string[i:]))
        ret.append(m.group(2))
        i = m.end(1)
        if i < len(string) and string[i] == ';':
            i += 1
            continue
        break

    for i in range(len(ret)):
        if ret[i].isdigit() or ret[i][0] == '-' and ret[i][1:].isdigit():
            ret[i] = int(ret[i])
        elif ret[i][0] == '"':
            try:
                ret[i] = six.text_type(ret[i][1:-1].replace(escape_char * 2,
                                                            escape_char))
                j = 0
                while j < len(ret[i]):
                    if (ret[i][j] == '<' and ret[i][j + 1] == 'U' and
                            ret[i][j + 6] == '>'):
                        ret[i] = (ret[i][0:j] +
                                  six.unichr(
                                      int(ret[i][(j + 2):(j + 6)], 16)) +
                                  ret[i][(j + 7):])
                    j += 1
            except UnicodeDecodeError:
                ret[i] = 'Someone put illegal non-ASCII here!'
        # else: <U0123> or <U01234567>
        # else: <U0123>..<U4567>
        # else: <TREMA>

    return ret


def parse_libc_localedata(filename, cache):
    if filename in cache:
        return cache[filename]
    cache[filename] = None  # catch broken recursion: will fail on .get access

    # The result
    ret = {}

    input = io.open(filename, 'r', encoding='utf-8')
    comment_char, escape_char = '#', '\\\\'
    category = None
    multiline_buffer = False

    re_cat_s = r'^(LC_[A-Z]+)(\s*Comment\s+.*)?$'
    re_cat = re.compile(re_cat_s.replace('Comment', comment_char), re.DOTALL)
    re_stmt = re.compile(r'^([A-Za-z_]+)\s+(.*?)$', re.DOTALL)
    re_comment_s = r'^Comment(\s+.*)?$'
    re_comment = re.compile(
        re_comment_s.replace('Comment', comment_char))
    re_midcomment_s = r'^(([^"]|"[^"]*")+)Comment.*(Escape)$'
    re_midcomment = re.compile(
        re_midcomment_s.replace(
            'Comment', comment_char).replace('Escape', escape_char))
    re_multiline_s = r'(^|[^Escape]|EscapeEscape)*Escape$'
    re_multiline = re.compile(
        re_multiline_s.replace('Escape',
                               escape_char * (int(escape_char == '\\') + 1)))

    for line in itertools.chain(input, ('\n',)):  # append LF to complete work
        # Treat as utf-8, ignore whitespace
        line = line.strip()
        # Drop trailing comments
        comm = re_midcomment.match(line)
        if comm:
            line = comm.group(1).rstrip() + comm.group(3)
        # Ignore comments (escape char on EOL does not "work")
        if re_comment.match(line):
            continue
        # If previous line was multiline, append
        if multiline_buffer:
            line = multiline_buffer + line.lstrip()
        # If line matches multiline, set multiline_buffer and restart
        if re_multiline.match(line):
            multiline_buffer = line[:-len(escape_char)]
            continue
        multiline_buffer = False

        # Ignore empty lines
        if line == '':
            pass
        # Check categories
        elif re_cat.match(line):
            m = re_cat.match(line)
            category = m.group(1)
            ret[category] = {}
        # Check statements
        elif re_stmt.match(line):
            m = re_stmt.match(line)
            keyword, args = m.group(1), m.group(2)
            # Special case where we close a category
            if keyword == 'END' and args == category:  # e.g. "END LC_NUMERIC"
                category = None
            # Skip over categories that we're not interested in
            elif (category is not None and
                    (CATEGORIES is not None and category not in CATEGORIES)):
                pass
            # Switch comment character
            elif keyword == 'comment_char':
                comment_char = args
                if args == '\\':
                    comment_char = '\\\\'
                re_cat = re.compile(
                    re_cat_s.replace('Comment', comment_char), re.DOTALL)
                re_comment = re.compile(
                    re_comment_s.replace('Comment', comment_char))
                re_midcomment = re.compile(
                    re_midcomment_s.replace(
                        'Comment', comment_char).replace(
                            'Escape', escape_char))
            # Switch escape character
            elif keyword == 'escape_char':
                escape_char = args
                if args == '\\':
                    escape_char = '\\\\'
                re_multiline = re.compile(
                    re_multiline_s.replace('Escape', escape_char))
                re_midcomment = re.compile(
                    re_midcomment_s.replace(
                        'Comment', comment_char).replace(
                            'Escape', escape_char))
            # Load other file
            elif keyword == 'copy':
                new_filename = os.path.join(
                    os.path.dirname(filename),
                    parse_args(args, escape_char)[0])
                recurse_data = parse_libc_localedata(new_filename, cache)
                if recurse_data is None:
                    raise ParseException(
                        'recursion during copy from %s to %s' % (
                            new_filename, filename))
                # Merge inner file data
                for copy_cat in (category,) if category else recurse_data:
                    if copy_cat not in ret:
                        ret[copy_cat] = {}
                    ret[copy_cat].update(recurse_data.get(copy_cat, {}))
            # Wow.. we even have defines...
            elif keyword == 'ifdef' or keyword == 'define':
                pass
            # Order start and end? Forget it...
            elif keyword == 'order_start':
                pass
            # Args.. store them
            else:
                assert args != ''
                args = parse_args(args, escape_char)
                # One-sized lists are not lists
                if len(args) == 1:
                    args = args[0]
                # Lists stay lists (even when empty)
                else:
                    # Trim tailing zeroes from lists (see
                    # locale.localeconv() output).
                    while len(args) >= 1 and args[-1] == 0:
                        args.pop()
                ret[category][keyword] = args
        # No match
        else:
            # Skip over uninteresting categories
            if (category is not None and
                    (CATEGORIES is not None and category not in CATEGORIES)):
                pass
            # We don't do defines
            elif keyword == 'else' or keyword == 'endif':
                pass
            # And we don't do order start and end.
            elif keyword == 'order_end':
                pass
            # We don't do anything with non-statements, really ;)
            else:
                # print('(unmatched: %s)' % (line,))
                pass

    # Purge empty categories
    for key in list(ret.keys()):
        if len(ret[key]) == 0:
            del ret[key]

    cache[filename] = ret
    return ret


def get_libc_sources(locale_path='/usr/share/i18n/locales'):
    file_match = re.compile(r'^[a-z]+_[A-Z]+$')

    sources = []
    for file in os.listdir(locale_path):
        if file_match.match(file):
            sources.append((file, os.path.join(locale_path, file)))
    sources.sort()
    return sources


def convert_all_libc_locales(src_path, dst_path):
    import json
    import sys

    languages = get_libc_sources(src_path)
#    languages = [('csb_PL', os.path.join(src_path, 'csb_PL'))]
#    print parse_args('"<U003B>"', '/')
#    sys.exit(1)

    print('Processing languages...')
    cache = {}
    for languagecode, filename in languages:
        # if languagecode != 'uk_UA':
        #     continue
        sys.stdout.write('\r%s\r%s' % (' ' * 16, languagecode))
        sys.stdout.flush()
        try:
            localedata = parse_libc_localedata(filename, cache)
            if localedata is None:
                raise ParseException('broken recursion?')
        except ParseException as e:
            sys.stdout.write('\n')
            sys.stderr.write('Language file %s corrupt: %s\n' % (
                filename, e))
        else:
            try:
                os.mkdir(os.path.join(dst_path, languagecode))
            except OSError:
                pass
            for category in localedata:
                dst_file = open(os.path.join(dst_path, languagecode, category),
                                'w', encoding='utf-8')
                json.dump(
                    obj=localedata[category], fp=dst_file,
                    sort_keys=True, separators=(',', ':'))
                dst_file.close()

    print('\r%s\r...done' % (' ' * 16,))


if __name__ == '__main__':
    convert_all_libc_locales(
        '/usr/share/i18n/locales',
        os.path.join(os.path.dirname(__file__), 'locale')
    )



import json
import sys
import pickle
for f in sys.argv[1:]:
    print(f)
    with open(f, 'rb') as fp:
        d = pickle.load(fp)
    with open(f, 'w') as fp:
        json.dump(obj=d, fp=fp, sort_keys=True, separators=(',', ':'))



#!/usr/bin/env python
# vim: set ts=8 sw=4 sts=4 et:
# ======================================================================
# Copyright (C) 2008-2016 Walter Doekes (wdoekes) at OSSO B.V.
# This file is part of Pyl10n.
#
# Pyl10n is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyl10n is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyl10n.  If not, see <http://www.gnu.org/licenses/>.
# ======================================================================
from __future__ import unicode_literals

from . import pyl10n_core as _p


def format(format, val, grouping=False, monetary=False, dutch_rounding=False,
           locale=None):
    if monetary:
        assert format == '%f'
        return currency(val, False, grouping, dutch_rounding=dutch_rounding,
                        locale=locale)

    assert len(format) and format[0] == '%'
    conv = _p.localeconv(locale)

    if dutch_rounding:
        val = dutch_round(val, _format_to_fractionals(format))
    ret = format % val
    if 'e' in ret or 'E' in ret:  # we're looking at exponents.. blame user
        return ret
    return _group_and_decimal(ret, grouping, conv['decimal_point'],
                              conv['thousands_sep'], conv['grouping'])


def currency(val, symbol=True, grouping=False, international=False,
             dutch_rounding=False, locale=None):
    conv = _p.localeconv(locale)
    neg = val < 0

    if neg:
        symbol_before = bool(conv['n_cs_precedes'])
        sign = conv['negative_sign']
        sep_by_space = bool(conv['n_sep_by_space'])
        positioning = conv['n_sign_posn']
    else:
        symbol_before = bool(conv['p_cs_precedes'])
        sign = conv['positive_sign']
        sep_by_space = bool(conv['p_sep_by_space'])
        positioning = conv['p_sign_posn']

    if international:
        fractionals = int(conv['int_frac_digits'])

        if symbol:
            if isinstance(symbol, bool):
                symbol_char = conv['int_curr_symbol']
            else:
                symbol_char = symbol
            if len(symbol_char) > 3:
                space_between_symbol_value = symbol_char[3]
            else:
                space_between_symbol_value = ''
            symbol_char = symbol_char[0:3]
    else:
        fractionals = int(conv['frac_digits'])

        if symbol:
            if isinstance(symbol, bool):
                symbol_char = conv['currency_symbol']
            else:
                symbol_char = symbol
            space_between_symbol_value = ('', ' ')[sep_by_space]

    if dutch_rounding:
        val = dutch_round(val, fractionals)
    val = abs(val)
    ret = ('%%.%if' % fractionals) % val
    ret = _group_and_decimal(ret, grouping, conv['mon_decimal_point'],
                             conv['mon_thousands_sep'], conv['mon_grouping'])

    if not symbol:
        if positioning == 0:
            return '(%s)' % (ret,)
        elif positioning == 1 or positioning == 3 or positioning == 127:
            return '%s%s' % (sign, ret)
        elif positioning == 2 or positioning == 4:
            return '%s%s' % (ret, sign)
        assert False

    if symbol_before:
        args = [symbol_char, space_between_symbol_value, ret]
    else:
        args = [ret, space_between_symbol_value, symbol_char]

    if positioning == 0:
        ret = '(%s%s%s)' % tuple(args)
    else:
        if (positioning == 1 or (positioning == 3 and symbol_before) or
                positioning == 127):
            args.insert(0, sign)
        elif positioning == 2 or (positioning == 4 and not symbol_before):
            args.append(sign)
        elif positioning == 3:
            args.insert(2, sign)
        elif positioning == 4:
            args.insert(1, sign)
        else:
            assert False
        ret = '%s%s%s%s' % tuple(args)

    return ret


def atof(string, allow_grouping=True, func=float, locale=None):
    conv = _p.localeconv(locale)
    if allow_grouping:
        ts = conv['thousands_sep']
        ds = conv['decimal_point']
        if '.' in string and '.' not in (ts, ds):
            raise ValueError('invalid decimal separator found')
        string = string.replace(ts, '')  # get rid of grouping (ignore pos)
    else:
        ds = conv['decimal_point']
        if '.' in string and '.' != ds:
            raise ValueError('invalid decimal separator found')
    if ds != '':
        string = string.replace(ds, '.')
    return func(string)


def atoi(string, *args, **kwargs):
    kwargs['func'] = int
    return atof(string, *args, **kwargs)


def dutch_round(val, fractionals):
    for i in range(fractionals):
        val *= 10.0
    leftovers = round(val % 1, 4)  # round because float fluctuates
    if leftovers == 0.5:
        val = int(round(val, 4))
        if val % 2 == 1:
            val += (1, -1)[val < 0]
    else:
        val = round(val, 0)
    for i in range(fractionals):
        val /= 10.0
    return val


def str(val, locale=None):
    return format('%.12g', val, locale=locale)


def _format_to_fractionals(format):
    try:
        i = format.index('.')
        for j in range(i + 1, len(format)):
            if format[j] not in '0123456789':
                fractionals = int(format[(i + 1):j])
                break
        else:
            fractionals = int(fractionals[i + 1])
    except ValueError:
        fractionals = 0
    return fractionals


def _group(val, group_char, group_list):
    if val[0] not in '0123456789':
        sign = val[0]
        val = val[1:]
    else:
        sign = ''

    ret = []
    i = len(val)
    # group_list defines grouping from right to left
    group = 127
    for group in group_list:
        if i <= 0 or group == 127:  # CHAR_MAX => no more grouping
            break
        # append next group to list
        ret.insert(0, val[max(0, i - group):i])
        i -= group
    # continue with last value from group_list
    while i > 0:
        ret.insert(0, val[max(0, i - group):i])
        i -= group
    # concat and return
    return sign + group_char.join(ret)


def _group_and_decimal(val, grouping, decimal_char, group_char, group_list):
    if grouping:
        if '.' in val:
            left, right = val.split('.')
            return '%s%s%s' % (_group(left, group_char, group_list),
                               decimal_char, right)
        else:
            return _group(val, group_char, group_list)
    return val.replace('.', decimal_char)


def pyl10n_numeric_test():
    # TODO: create tests :)
    pass



#!/usr/bin/env python
# vim: set ts=8 sw=4 sts=4 et:
# ======================================================================
# Copyright (C) 2008-2016 Walter Doekes (wdoekes) at OSSO B.V.
# This file is part of Pyl10n.
#
# Pyl10n is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyl10n is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyl10n.  If not, see <http://www.gnu.org/licenses/>.
# ======================================================================
from __future__ import unicode_literals

import re
import time
from . import pyl10n_core as _p
_percent_re = re.compile(r'(%.|[^%]+)')  # misses modifiers (like %Ey)


def format_date(date, locale=None):
    lc_time = _p.localeconv_by_category('LC_TIME', locale)
    return _strftime(lc_time['d_fmt'], lc_time, date)


def format_datetime(date, locale=None):
    lc_time = _p.localeconv_by_category('LC_TIME', locale)
    return _strftime(lc_time['d_t_fmt'], lc_time, date)


def format_time(date, locale=None):
    lc_time = _p.localeconv_by_category('LC_TIME', locale)
    return _strftime(lc_time['t_fmt'], lc_time, date)


def strftime(fmt, date=None, locale=None):
    if date is None:
        import datetime
        date = datetime.datetime.now()
    lc_time = _p.localeconv_by_category('LC_TIME', locale)
    return _strftime(fmt, lc_time, date)


def _strftime(fmt, lc_time, date=None):
    lt = date.timetuple()  # local time

    matches = _percent_re.findall(fmt)
    for i, v in enumerate(matches):
        if v[0] != '%':
            continue
        c = v[1]  # len(v) == 2 according to _percent_re

        if c == '%': matches[i] = '%'
        elif c == 'a': matches[i] = lc_time['abday'][(lt[6] + 1) % 7]
        elif c == 'A': matches[i] = lc_time['day'][(lt[6] + 1) % 7]
        elif c == 'b' or c == 'h': matches[i] = lc_time['abmon'][lt[1] - 1]
        elif c == 'B': matches[i] = lc_time['mon'][lt[1] - 1]
        elif c == 'c': matches[i] = _strftime(lc_time['d_t_fmt'], lc_time,
                                              date)
        elif c == 'C': matches[i] = '%02d' % (lt[0] / 100)
        elif c == 'd': matches[i] = '%02d' % lt[2]
        elif c == 'D': matches[i] = ('%02d/%02d/%02d' %
                                     (lt[1], lt[2], lt[0] % 100))
        elif c == 'e': matches[i] = '%2d' % lt[2]
        # %E => modifier, not implemented
        elif c == 'F': matches[i] = '%d-%02d-%02d' % (lt[0], lt[1], lt[2])
        # %G => NOT IMPLEMENTED
        # %g => NOT IMPLEMENTED
        elif c == 'H': matches[i] = '%02d' % lt[3]
        elif c == 'I': matches[i] = '%02d' % (((lt[3] + 11) % 12) + 1)
        elif c == 'j': matches[i] = '%03d' % lt[7]
        elif c == 'k': matches[i] = '%2d' % lt[3]
        elif c == 'l': matches[i] = '%2d' % (((lt[3] + 11) % 12) + 1)
        elif c == 'm': matches[i] = '%02d' % lt[1]
        elif c == 'M': matches[i] = '%02d' % lt[4]
        elif c == 'n': matches[i] = '\n'
        # %O => modifier, not implemented
        elif c == 'p': matches[i] = lc_time['am_pm'][lt[3] >= 12]
        elif c == 'P': matches[i] = lc_time['am_pm'][lt[3] >= 12].lower()  # (GNU)
        elif c == 'r': matches[i] = _strftime(lc_time['t_fmt_ampm'], lc_time,
                                              date)
        elif c == 'R': matches[i] = '%02d-%02d' % (lt[3], lt[4])
        elif c == 's': matches[i] = str(time.mktime(date.utcnow()))
        elif c == 'S': matches[i] = '%02d' % lt[5]
        elif c == 't': matches[i] = '\t'
        elif c == 'T': matches[i] = '%02d:%02d:%02d' % (lt[3], lt[4], lt[5])
        elif c == 'u': matches[i] = str(lt[6] + 1)
        # %U => NOT IMPLEMENTED
        # %V => NOT IMPLEMENTED
        elif c == 'w': matches[i] = str((lt[6] + 1) % 7)
        # %W => NOT IMPLEMENTED
        elif c == 'x': matches[i] = _strftime(lc_time['d_fmt'], lc_time, date)
        elif c == 'X': matches[i] = _strftime(lc_time['t_fmt'], lc_time, date)
        elif c == 'y': matches[i] = '%02d' % (lt[0] % 100)
        elif c == 'Y': matches[i] = str(lt[0])
        # %z => NOT IMPLEMENTED # (GNU)
        # %Z => NOT IMPLEMENTED
        elif c == 'z' or c == 'Z': matches[i] = ''  # BROKEN, NOT IMPLEMENTED
        # %+ => NOT IMPLEMENTED

    return ''.join(matches).strip()


def pyl10n_time_test():
    import datetime
    now = datetime.datetime.now()
    print(format_datetime(now - datetime.timedelta(minutes=5), 'de_DE'))
    print(format_datetime(now - datetime.timedelta(minutes=10), 'en_US'))
    print(format_datetime(now - datetime.timedelta(minutes=15), 'nl_NL'))
    print(format_datetime(now - datetime.timedelta(minutes=20), 'sv_SE'))



#!/usr/bin/env python
# vim: set ts=8 sw=4 sts=4 et:
# ======================================================================
# Copyright (C) 2009-2016 Walter Doekes (wdoekes) at OSSO B.V.
# This file is part of Pyl10n.
#
# Pyl10n is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyl10n is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyl10n.  If not, see <http://www.gnu.org/licenses/>.
# ======================================================================
from __future__ import unicode_literals

import re
from . import pyl10n_core as _p
_percent_re = re.compile(r'(%.|[^%]+)')


def teldom2string(phone_tuple, locale=None):
    '''
    See _tel2string help.
    '''
    lc_telephone = _p.localeconv_by_category('LC_TELEPHONE', locale)
    return _tel2string(
        phone_tuple,
        lc_telephone.get('tel_dom_fmt',
                         lc_telephone.get('tel_int_fmt', '+%c %a%t%l')),
        lc_telephone)


def telint2string(phone_tuple, locale=None):
    '''
    See _tel2string help.
    '''
    lc_telephone = _p.localeconv_by_category('LC_TELEPHONE', locale)
    return _tel2string(
        phone_tuple,
        lc_telephone.get('tel_int_fmt', '+%c %a%t%l'),
        lc_telephone)


def _tel2string(phone_tuple, fmt, locale_dict=None):
    '''
    See: ISO/IEC WD 15435

    We do not know the country codes or area codes, so we'll take the phone
    number as a tuple of ([country_code, [area_code,]] local_number). The
    country code shouldn't have any prefixes (like + or 0), nor should the
    area code.

    We don't do anything with int_prefix (country-code) or int_select
    (international-prefix). The nat_select (national-prefix) does not exist
    so we assume it to be 0.

    %a area code without nationwide prefix (prefix is often <0>).
    %A area code including nationwide prefix (prefix is often <0>).
    %l local number (within area code).
    %e extension (to local number)
    %c country code
    %C alternate carrier service code used for dialling abroad
    %t Insert a <space> if the previous descriptor's value was not an empty
       string; otherwise ignore.
    '''
    if len(phone_tuple) == 3:
        country_code, area_code, local_number = (str(i) for i in phone_tuple)
    elif len(phone_tuple) == 2:
        country_code = str(phone_tuple[0])
        area_code = ''
        local_number = str(phone_tuple[1])

    # # Fill in a bit of area code.. two digits sounds about right ;)
    # if area_code == '' and len(local_number) > 3:
    #     area_code, local_number = local_number[0:2], local_number[2:]

    nat_select = '0'

    matches = _percent_re.findall(fmt)
    last = ''
    for i, v in enumerate(matches):
        if v[0] != '%':
            continue
        c = v[1]  # len(v) == 2 according to _percent_re

        if c == '%':
            last = matches[i] = '%'
        elif c == 'a':
            last = matches[i] = area_code
        elif c == 'A':
            last = matches[i] = '%s%s' % (nat_select, area_code)
        elif c == 'l':
            last = matches[i] = local_number
        elif c == 'e':
            raise NotImplementedError('Was not expecting extension in format',
                                      fmt)
        elif c == 'c':
            last = matches[i] = country_code
        elif c == 'C':
            raise NotImplementedError('Was not expecting unknown C in format',
                                      fmt)
        elif c == 't':
            last, matches[i] = '', ('', ' ')[last != '']
        else:
            raise ValueError('Unknown field descriptor in format', fmt)

    return ''.join(matches)


def pyl10n_telephone_test():
    print(teldom2string((31, 50, 1234567), 'en_US'))
    print(telint2string((31, 50, 1234567), 'en_US'))
    print(teldom2string((31, 50, 1234567), 'en_GB'))
    print(telint2string((31, 50, 1234567), 'en_GB'))
    print(teldom2string((31, 50, 1234567), 'nl_NL'))
    print(telint2string((31, 50, 1234567), 'nl_NL'))
    print(teldom2string((31, 50, 1234567), 'sv_SE'))
    print(telint2string((31, 50, 1234567), 'sv_SE'))



#!/usr/bin/env python
from distutils.core import setup
import io

if __name__ == '__main__':
    long_descriptions = []
    with io.open('README.rst', encoding='utf-8') as file:
        long_descriptions.append(file.read())

    setup(
        name='pyl10n',
        version='1.3',
        packages=['pyl10n'],
        # Include all locale non-py files in pyl10n as well.
        package_data={'pyl10n': ['locale/README', 'locale/*/LC_*']},
        # Install into '.../share/doc/pyl10n' and include it so we can
        # open it during install.
        data_files=[('share/doc/pyl10n', ['README.rst'])],
        description='Pyl10n is a localization (l10n) library for python',
        long_description=(u'\n\n\n'.join(long_descriptions)),
        author='Walter Doekes, OSSO B.V.',
        author_email='wjdoekes+pyl10n@osso.nl',
        url='https://github.com/ossobv/pyl10n',
        license='LGPLv3+',
        platforms=['linux'],
        # https://pypi.python.org/pypi?%3Aaction=list_classifiers
        classifiers=[
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            ('License :: OSI Approved :: GNU Lesser General Public License v3 '
             'or later (LGPLv3+)'),
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Topic :: Software Development :: Localization',
            'Topic :: Software Development :: Libraries',
        ],
    )

# vim: set ts=8 sw=4 sts=4 et ai tw=79:
